/***************************************************************************
                          projectmanager.h  -  description
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROJECTMANAGER_H
#define PROJECTMANAGER_H

#include <qobject.h>
#include <qstring.h>
#include <qdir.h>
#include <kconfig.h>
#include <qlistview.h>

/**
 * Abstract base class for classes that need the list of project files.
 * Objects of classes derived from this one are used as a parameter to
 * ProjectManager::fillList(), which reads all file entries in the project,
 * and calls addItem() for each.
 * Any class that wishes to retrieve the project's file list, should inherit
 * from this class, and implement addItem().
 * @author Elad Lahav
 */

class FileListTarget
{
public:
	/**
	 * Class constructor.
	 */
	FileListTarget() {}

	/**
	 * Appends a file to the list.
	 * @param	sFilePath	The full path of the file to add
	 */
	virtual void addItem(const QString& sFilePath) = 0;
};

/**
 * Abstract base class for classes that need the list of project files.
 * Objects of classes derived from this one are used as a parameter to
 * ProjectManager::writeList(), which calls getFirstItem() and getNextItem(),
 * and writes the returned values to the project's 'cscope.files' file.
 * Any class that wishes to retrieve the project's file list, should inherit
 * from this class, and implement firstItem() and nextItem().
 * @author Elad Lahav
 */

class FileListSource
{
public:
	/**
	 * Class constructor.
	 */
	FileListSource() {}

	/**
	 * Returns the first file in the list, and initiates a new iteration.
	 * @param	sFilePath	Holds the path of the first file, upon return
	 * @return	true if there are more files, false otherwise
	 */
	virtual bool firstItem(QString& sFilePath) = 0;

	/**
	 * Returns the next file in the list.
	 * @param	sFilePath	Holds the path of the file, upon return
	 * @return	true if there are more files, false otherwise
	 */
	virtual bool nextItem(QString& sFilePath) = 0;
};

/**
 * Maintains a KScope project.
 * A project is a set of source files, on which to run Cscope, as well as
 * other programmes. The project is maintained using a set of configuration
 * files, located in a common directory. These files include:
 * - cscope.proj: A KDE configuration file that stores project parameters
 * - cscope.files: Holds the list of source files included in this project
 * To these cscope adds its symbol reference files.
 * The ProjectManager class is responsible for creating new projects, adding
 * and removing source files, and allowing other objects access to the
 * currently active project (such as filling a list object with the source 
 * files).
 * @author Elad Lahav
 */

class ProjectManager : public QObject
{
	Q_OBJECT
	
public: 
	ProjectManager();
	virtual ~ProjectManager();

	bool create(const QString&, const QString&, bool, bool, 
		const QStringList&);
	bool open(const QString&);
	bool openCscopeOut(const QString&);
	void close();
	int addFiles(const QStringList&);
	void fillList(FileListTarget*);
	void writeList(FileListSource*);
	
	bool isEmpty();
	QString getFileTypes();
	void setFileTypes(const QStringList&);
	void getOptions(bool&, bool&);
	void getOptions(QStringList&);
	void setOptions(bool, bool);
	void getOpenFiles(QStringList&);
	void setOpenFiles(QStringList&);
	void getQueryFiles(QStringList&);
	void setQueryFiles(QStringList&);
	QString getRoot();
	void setRoot(const QString&);

	/**
	 * Determines whether a project is open, i.e., a project directory is
	 * associated with this object.
	 * @return	true if the project is open, false otherwise
	 */
	bool isOpen() { return m_bOpen; }

	/**
	 * Determines whether a project is based on a Cscope.out file, and is
	 * therefore considered as a temporary project.
	 * @return	true if this is a temporary project, false otherwise
	 */
	bool isTemporary() { return m_bTempProj; }

	/**
	 * @return	The name of the current project
	 */
	QString getName() { return m_sName; }
	
	/**
	 * @return	The full path of the project's directory
	 */
	QString getPath() { return m_dir.absPath(); }
	
signals:
	/**
	 * Emitted when a set of files is added to the current project.
	 * @param	slFiles	A list containing the full paths of the new files
	 */
	void filesAdded(const QStringList& slFiles);
	
	/**
	 * Emitted when the list of files in a project is modified.
	 */
	void fileListChanged();
	
private:
	/** The name of the project, as written in the configuration file */
	QString m_sName;

	/** The directory associated with the project */
	QDir m_dir;

	/** The file that holds the paths of all source files in this project
		("cscope.files") */
	QFile* m_pFiles;

	/** The configuration file ("cscope.proj") */
	KConfig* m_pConf;

	/** true if there is an active project, false otherwise */
	bool m_bOpen;
	
	/** true if the current project is a temporary wrapper for a Cscope.out
		file */
	bool m_bTempProj;
	
	static void initConfig(const QString&, QFile**, KConfig**);
};

#endif
